/*

 Copyright 2013, 2014 SIANI - ULPGC
 Jose Juan Hernandez Cabrera
 Jose Evora Gomez
 Johan Sebastian Cortes Montenegro
 Maria del Carmen Sánchez Medrano

 This File is Part of JavaFMI Project

 JavaFMI Project is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.

 JavaFMI Project is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with JavaFMI Library.  If not, see <http://www.gnu.org/licenses/>.

 */
 package org.javafmi.kernel.v2.nativeaccess;

import com.sun.jna.Library;
import com.sun.jna.Pointer;
import com.sun.jna.Structure;
import com.sun.jna.ptr.PointerByReference;

import java.nio.ByteBuffer;
import java.nio.DoubleBuffer;
import java.nio.IntBuffer;

public interface FmiV2Interface extends Library {
    public static final int FMI_CO_SIMULATION = 1;

    Pointer fmiInstantiate(
            String modelName,
            int fmiType,
            String guID,
            String resourcesLocation,
            Structure.ByReference callbacks,
            boolean visible,
            boolean loggingOn);

    void fmiFreeInstance(Pointer component);

    int fmiSetDebugLogging(Pointer component, boolean loggingOn, int numberOfCategories, String[] categories);

    int fmiSetupExperiment(Pointer component, boolean toleranceDefined, double tolerance, double startTime, boolean stopTimeDefined, double stopTime);

    int fmiEnterInitializationMode(Pointer component);

    int fmiExitInitializationMode(Pointer component);

    int fmiTerminate(Pointer component);

    int fmiReset(Pointer component);

    void fmiGetReal(Pointer component, int[] valueReferences, int numberOfValueReferences, DoubleBuffer realValues);

    void fmiGetInteger(Pointer component, int[] valueReferences, int numberOfValueReferences, IntBuffer intValues);

    void fmiGetBoolean(Pointer component, int[] valueReferences, int numberOfValueReferences, IntBuffer booleanValue);

    void fmiGetString(Pointer component, int[] valueReferences, int numberOfValueReferences, PointerByReference stringValue);

    int fmiSetReal(Pointer component, int[] valueReferences, int numberOfValueReferences, DoubleBuffer realValues);

    int fmiSetInteger(Pointer component, int[] valueReferences, int numberOfValueReferences, IntBuffer intValues);

    int fmiSetBoolean(Pointer component, int[] valueReferences, int numberOfValueReferences, IntBuffer booleanValues);

    int fmiSetString(Pointer component, int[] valueReferences, int numberOfValueReferences, PointerByReference stringValues);

    int fmiGetFMUstate(Pointer component, PointerByReference pointerToState);

    int fmiSetFMUstate(Pointer component, Pointer state);

    int fmiFreeFMUstate(Pointer component, PointerByReference state);

    void fmiSerializedFMUstateSize(Pointer component, Pointer state, IntBuffer stateSize);

    int fmiSerializeFMUstate(Pointer component, Pointer state, ByteBuffer serializedState, int stateSize);

    int fmiDeSerializeFMUstate(Pointer component, byte[] serializedState, int stateSize, PointerByReference pointerToState);

    void fmiGetDirectionalDerivative(Pointer component, int[] unknownVariablesValueReferences, int numberOfUnknowns, int[] knownVariablesValueReferences, int numberOfKnowns, double[] knownDifferential, DoubleBuffer unknownDifferential);

    int fmiDoStep(Pointer component, double currentSimulationTime, double stepSize, boolean noSetFmuStatePriorToCurrentTime);

    int fmiCancelStep(Pointer component);
}
