/*

 Copyright 2013, 2014 SIANI - ULPGC
 Jose Juan Hernandez Cabrera
 Jose Evora Gomez
 Johan Sebastian Cortes Montenegro
 Maria del Carmen Sánchez Medrano

 This File is Part of JavaFMI Project

 JavaFMI Project is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.

 JavaFMI Project is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with JavaFMI Library.  If not, see <http://www.gnu.org/licenses/>.

 */
 package org.javafmi.kernel.v2rc2.nativeaccess;

import com.sun.jna.Library;
import com.sun.jna.Pointer;
import com.sun.jna.Structure;
import com.sun.jna.ptr.PointerByReference;

import java.nio.ByteBuffer;
import java.nio.DoubleBuffer;
import java.nio.IntBuffer;

public interface FmiV2RC2Interface extends Library {
    public static final int FMI_CO_SIMULATION = 1;

    Pointer fmi2Instantiate(String modelName, int fmiType, String guID, String resourcesLocation, Structure byReference, boolean visible, boolean loggingOn);

    void fmi2FreeInstance(Pointer component);

    int fmi2SetDebugLogging(Pointer component, boolean loggingOn, int numberOfCategories, String[] categories);

    int fmi2SetupExperiment(Pointer component, boolean toleranceDefined, double tolerance, double startTime, boolean stopTimeDefined, double stopTime);

    int fmi2EnterInitializationMode(Pointer component);

    int fmi2ExitInitializationMode(Pointer component);

    int fmi2Terminate(Pointer component);

    int fmi2Reset(Pointer component);

    void fmi2GetReal(Pointer component, int[] valueReferences, int numberOfValueReferences, DoubleBuffer realValues);

    void fmi2GetInteger(Pointer component, int[] valueReferences, int numberOfValueReferences, IntBuffer intValues);

    void fmi2GetBoolean(Pointer component, int[] valueReferences, int numberOfValueReferences, IntBuffer booleanValue);

    void fmi2GetString(Pointer component, int[] valueReferences, int numberOfValueReferences, PointerByReference stringValue);

    int fmi2SetReal(Pointer component, int[] valueReferences, int numberOfValueReferences, DoubleBuffer realValues);

    int fmi2SetInteger(Pointer component, int[] valueReferences, int numberOfValueReferences, IntBuffer intValues);

    int fmi2SetBoolean(Pointer component, int[] valueReferences, int numberOfValueReferences, IntBuffer booleanValues);

    int fmi2SetString(Pointer component, int[] valueReferences, int numberOfValueReferences, PointerByReference stringValues);

    int fmi2GetFMUstate(Pointer component, PointerByReference pointerToState);

    int fmi2SetFMUstate(Pointer component, Pointer state);

    int fmi2FreeFMUstate(Pointer component, Pointer state);

    void fmi2SerializedFMUstateSize(Pointer component, Pointer state, IntBuffer stateSize);

    int fmi2SerializeFMUstate(Pointer component, Pointer state, ByteBuffer serializedState, int stateSize);

    int fmi2DeSerializeFMUstate(Pointer component, byte[] serializedState, int stateSize, PointerByReference pointerToState);

    void fmi2GetDirectionalDerivative(Pointer component, int[] unknownVariablesValueReferences, int numberOfUnknowns, int[] knownVariablesValueReferences, int numberOfKnowns, double[] knownDifferential, DoubleBuffer unknownDifferential);

    int fmi2DoStep(Pointer component, double currentSimulationTime, double stepSize, boolean noSetFmuStatePriorToCurrentTime);

    int fmi2CancelStep(Pointer component);
}
